//
//  Copyright (c) 1993-2002 Easysoft Ltd. All rights reserved.
//
//  Sample java client for the Easysoft XML-ODBC Server.
//
//  $Id: XmlOdbc.java,v 1.8 2002/07/09 13:20:56 martin Exp $
//
//
//
import java.net.*;
import java.io.*;
import java.util.*;
/**
 *
 * The XmlOdbc class is used to communicate with the Easysoft XML-ODBC
 * Server.
 *
 * <P>It allows you to send SQL statements to an ODBC driver
 * on the same machine as the XML-ODBC Server and retrieve the results
 * in XML. The protocol used is simple and documented by the DTDs
 * distributed with the XML-ODBC Server.
 * 
 */
public class XmlOdbc
{
    Socket                      soc;
    DataOutputStream            sout;
    DataInputStream             sin;

    /**
     *
     */
    public XmlOdbc (){};
    
    /**
     * Establishes socket connection to the XML-ODBC Server; 
     * Port defaults to 8895.
     *
     * @param server XML-ODBC Server machine
     *
     */
    public void connect(String server)
	throws   java.net.UnknownHostException, java.io.IOException
    {
       connect(server, 8895);
    }
    
    /**
     * Establishes socket connection to the XML-ODBC Server
     *
     * @param server XML-ODBC Server machine
     * @param port   Port on which the XML-ODBC Server is listening
     *
     */
    public void connect(String server, int port) 
	throws   java.net.UnknownHostException, java.io.IOException {
	soc = new Socket( server, port );
        sin = new DataInputStream( soc.getInputStream());
        sout = new DataOutputStream(soc.getOutputStream());
    }


    /**
     * Construct a request in XML and send it to the server.
     * Establish connection to an ODBC data source on the machine on which
     * the XML-ODBC Server is running using the connection string supplied,
     * and executes given SQL. The result set generated can be retrieved
     * as a string using getStringResponse() or as an input stream using
     * getStreamResponse()
     *
     * @param sql SQL query to be sent to the XML-ODBC Server
     * @param odbcConnectionString connection string 
     */
    public void sendRequest(String sql, String odbcConnectionString) throws Exception {

	String xml;

        xml = "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n";

        xml += "<Request>\n";
        xml += "<ConnectionString>" + odbcConnectionString + "</ConnectionString>\n";
        xml += "<Sql>" + sql + "</Sql>\n";
        xml += "<Format>\n";
        xml += "<Long/>\n";
        xml += "</Format>\n";
        xml += "</Request>\n";
	sendXml(xml);
    }

    
    /**
     * Get XML response from the server as a String
     */
    public String getStringResponse()
        throws Exception
    {
        int c;
        
        StringBuffer output = new StringBuffer(4096);
	int i = 0;
        while(( c = sin.read()) != -1 )
	    {   	

		output.append(( char ) c);
		//	 if (i>100)	System.out.println(output.substring(i-9,i));
		if (i>10 &&  output.substring(i-9,i).equals("</Result>"))
		    return output.toString();
		i++;
	    }
        return output.toString();
    } 

    /**
     * Get XML response from the server as an input stream that can be passed
     * to a parser.
     */
    public DataInputStream getStreamResponse()
        throws Exception
    {
        return sin;
    }

    /**
     * Send XML request to the server given as a String parameter
     *
     * @param xmlrequest request in XML, see DTDs.
     */
    public void sendXml(String xmlrequest)
        throws Exception
    {
        byte [] b = xmlrequest.getBytes();
        sout.write( b, 0, xmlrequest.length());
    }
}
