//
//  Copyright (c) 1993-2002 Easysoft Ltd. All rights reserved.
//
//  Sample java demo for the Easysoft XML-ODBC Server.
//
//  To use:
//
//  Compile with javac
//
//  Run with:
//
//  java XmlOdbcDemo -h server-host
//       [-p server_port]
//       [-s "SQL Query" -c "connection string"] | [-f request_file]
//
//  e.g. 
//  java XmlOdbcDemo -h myserver -p 8895 
//               -s "select * from table"
//               -c "DSN=test;UID=dbname;PWD=dbpassword;"
//
//  $Id: XmlOdbcDemo.java,v 1.3 2002/07/09 09:31:59 martin Exp $
//

import java.io.*;
public class XmlOdbcDemo
{
    
    public static void main( String[] args )
    {
        String                      server;
        String                      connect;
        String                      infile;
        String                      sql;
        int                         port;
        String                      xml="";


        port = 8895;
        server = "localhost";
        connect = "";
        infile = "";
        sql = "";

        if (args.length == 0) {
            System.err.println( "Usage: XmlOdbcDemo -h server [-p port] -c connect_string -f query_from_file -s sql ");
            System.exit( 1 );
        }
        
        for ( int i = 0; i < args.length; i++ ) 
        {
            if ( args[ i ].equals( "-h" ))
            {
                server = args[ ++i ];
            }
            else if ( args[ i ].equals( "-p" ))
            {
                port = new Integer( args[ ++i ] ).intValue();
            }
            else if ( args[ i ].equals( "-c" ))
            {
                connect = args[ ++i ];
            }
            else if ( args[ i ].equals( "-f" ))
            {
                infile = args[ ++i ];
            }
            else if ( args[ i ].equals( "-s" ))
            {
                sql = args[ ++i ];
            }
            else
            {
                System.err.println( "Usage: XmlOdbcDemo -h server [-p port] -c connect_string -f query_from_file -s sql ");
                System.exit( 1 );
            }
        }

        if ( sql.length() == 0 )
        {
            if ( infile.length() == 0 )
            {
                System.err.println( "Error: Either a query string or query file must be specified");
                System.exit( 2 );
            }

            try 
            {
                FileReader file = new FileReader( infile );
                int c;
                
                while(( c = file.read()) != -1 )
                {
                    xml += (char) c;
                }
            }
            catch( Exception e )
            {
                System.err.println( "Exception opening " + infile + " : " + e.getMessage());
                System.exit( 4 );
            }
        }
        else
        {
            if ( connect.length() == 0 )
            {
                System.err.println( "Error: A connection string is required with a query string");
                System.exit( 5 );
            }

        }
        try {
            XmlOdbc xo = new XmlOdbc();
            String result;
            xo.connect(server,port);
            if (xml.length() != 0) {
		xo.sendXml(xml);
		result = xo.getStringResponse();
	    }
            else {
                xo.sendRequest(sql, connect);
		result = xo.getStringResponse();
		}
            
            System.out.println(result);
        }
        catch (Exception e) {
            System.err.println(e.getMessage());
            System.exit(6);
        }
        
    }
}


        
