#!/usr/bin/perl -w
#
# Copyright (c) 1993-2002 Easysoft Ltd. All rights reserved.
#
# $Id: xmlodbc_example.pl,v 1.3 2002/07/04 11:46:46 martin Exp $
#
# This is just an example of using ESXMLODBC::XMLREQUEST.
# The ConnectionString, Server, Port and SQL are all hard-wired at the
# start of this script and the result-set is output twice using two different
# methods.
#
# This code does work but only if you change the hard-wired strings.
# If you just want to issue requests to the XML-ODBC server and see the
# results use xmlodbc_client.pl
#
# NOTE:
# [1] Make sure XMLREQUEST.pm is on your INC path.
# [2] You NEED to install XML::Parser from cpan to run this entire example.
#
# Output is to stdout.
#
use 5.005;
use strict;
use ESXMLODBC::XMLREQUEST;

use constant XMLODBC_SERVER =>"gambantein.easysoft.com";
use constant XMLODBC_PORT =>"8895";
use constant XMLODBC_CONNSTR => "DSN=demo;UID=demo;PWD=easysoft;";
use constant XMLODBC_SQL => "select * from oobdist_contact";

#
# New instance of XMLREQUEST
#
my $xoh = ESXMLODBC::XMLREQUEST->new();
#
# Connect to the XML-ODBC Server.
# We either set the Server/Port key in the hash before calling Connect
# or pass them in as an anonymous hashref to the Connect method.
#
# $xoh->{Server} = XMLODBC_SERVER;
# $xoh->{Port} = XMLODBC_PORT; - optional - the default anyway.
#
if (!$xoh->Connect({Server => XMLODBC_SERVER, Port => XMLODBC_PORT})) {
    print "$xoh->{ErrorState} - $xoh->{ErrorText}\n";
    exit 1;
}
#
# Execute some SQL on the specified datasource.
# Here again, we either Set ConnectionString, Truncate and Parse explicitly
# in the hash before calling Execute or pass them as an anonymous hashref to
# Execute.
# You may pass a single SQL statement as the first arg to Execute OR
# you set $xoh->{Sql} to be an reference to an array of SQL statements
# to be executed as a transaction.
#
# $xoh->{ConnectionString} = XMLODBC_CONNSTR;
# $xoh->{Parse} = 1; - the default
#   Parse the resulting XML and transform into an array of hash references
#   so you can use {Rows}, {NumRows} and {NumColumns}.
# $xoh->{Truncate} = 0;
#   Truncate SQL_CHAR columns that end in spaces.
#
if (!$xoh->Execute(XMLODBC_SQL, 
		    {ConnectionString => XMLODBC_CONNSTR,
		 Parse=>"1", Truncate=>0})) {
    print "$xoh->{ErrorState} - $xoh->{ErrorText}\n";
    exit 1;
}
#
# If you want to see the XML request sent to the server use
# print $xoh->{Request};
#

#
# You should now have the XML output in $xoh->{XML}
#
# print $xoh->{XML};
#

#
# If the XML was parsed (see Parse) then NumRows and NumColumns will be
# set to the number of rows in the result-set and the number of columns in
# each row.
#
print "Number of Rows: $xoh->{NumRows}\n";
print "Number of Columns: $xoh->{NumColumns}\n";
print "ErrorState: $xoh->{ErrorState}\n";
print "ErrorText: $xoh->{ErrorText}\n";

#
# Loop through the result-set printing it out.
#
# There are two methods to do this.
#
# Rows is an array (one element for each row) of hashrefs. Each hashref
# contains keys based on the column names.
#  
#
my $nrow = 0;
for ($nrow = 0; $nrow < $xoh->{NumRows}; $nrow++) {
    my $rowhash = $xoh->{Rows}[$nrow];
    if ($nrow == 0) {
	print join(",", keys %$rowhash), "\n"; # print column names
    }
    # print columns
    my $column;
    foreach $column (keys %$rowhash) {
	print $rowhash->{$column}, " ";
    }
    print "\n";
}

#
# Second method of returning the result set is using First, Next and Last.
#
my $hr;
if (!($hr = $xoh->First())) {
    print "$xoh->{ErrorState} - $xoh->{ErrorText}\n";
    exit 1;
}
print join (",", keys %$hr), "\n"; # print column names
while($hr) {
    my $col;
    foreach $col (keys %$hr) {
	print $hr->{$col}, " ";
    }
    print "\n";
    $hr = $xoh->Next();
}
$xoh->Disconnect();
