<?
//  XMLODBC client class
// (c) 1993-2002 Easysoft Limited
// 
// $Id: XMLODBC.inc,v 1.7 2002/07/15 15:15:40 martin Exp $
//
// main XMLODBC class
//
// You can use this class as follows:
//
// $x = new XMLODBC;
// $x->Connect(server, port);                  // connect to server
//   Or
//   $x->Hostname = "server";                  // where XML-ODBC Server is
//   $x->Port = 8895;                          // port server is listening on
//   $x->Connect();
// $x->Execute(connection_string, sql);        // execute SQL or transaction
//   Or
//   $x->ConnectionString = "connection string";
//   $x->SQL = "SQL statement";
//   Or
//     $x->SQL[0] = "insert/update/delete statement one of transaction";
//     $x->SQL[1] = "insert/update/delete statement two of transaction";
//     This executes statement 1 and 2 as a transaction.
// $x->Disconnect();                           // disconnect from server
//
// $x->TruncateTrailingBlanks
//   Boolean, defaults to FALSE.
//   If set causes the request to the server to be modified to ask the server
//   to truncate all trailing blanks from SQL_CHAR columns.
//
// $x->Request
//   This is the XML request issued by XMLODBC to the server.
//
// $x->XML
//   This is the XML returned from the server.
//
// $x->Parse
//   If set to true causes the returned XML to be parsed with expat.
//   This makes available NumRows, NumColumns, First, Next, Last and
//   Rows.
//
// $x->NumRows
//   If Parse=True and in an executed state then this is the number of rows
//   in the result-set.
//
// $x->NumColumns
//   If Parse=True and in an executed state then this is the number of columns
//   in each row in the result-set.
//
// $x->Rows
//   If Parse=True and in an executed state then this is an array of
//   associated arrays. The first dimension is the rows in the result-set.
//   The second dimension are associative arrays when the column names are
//   the keys.
//   e.g.
//     $p = $x->Row[1]; // row 2 - this is an associative array
//     print_r(array_keys ($p)); // prints the keys which are column names
//
// $x->First
//   If Parse=True and in an executed state then this returns the first
//   row (see Rows) of the result-set.
//
// $x->Next
//   If Parse=True and in an executed state then this returns the next
//   row (see Rows) of the result-set.
//
// $x->Last
//   If Parse=True and in an executed state then this returns the last
//   tow (see Rows) of the result-set.
//
class XMLODBC {
    var $Server;                                /* server to connect to */
    var $Port;                                  /* port to connect to */
    var $ConnectionString;                      /* ODBC connection string */
    var $SQL;                                   /* SQL statement */
    var $XML;                                   /* returned XML */
    var $Request;                               /* XML request issued */
    var $NumRows;                               /* # of rows in result-set */
    var $NumColumns;                            /* # of columns in a row */
    var $ErrorState;                            /* Error state string */
    var $ErrorText;                             /* Error text */
    var $Parse;
    var $Rows;
    
    // private
    var $_connected;
    var $_socket;
    var $_parser;
    var $_current_column;
    var $_current_element;
    var $_executed;
    var $_pos;
    
    function XMLODBC()
    {
        $this->ConnectionString = "";
        $this->Port = "8895";
        $this->HostName = "";
        $this->XML = "";
        $this->Request = "";
        $this->NumRows = 0;
        $this->NumColumns = 0;
        $this->ErrorState = "00000";
        $this->ErrorText = "";
        $this->TruncateTrailingBlanks = FALSE;
        $this->Parse = TRUE;
        
        $this->_connected = FALSE;
        $this->_executed = FALSE;

	//
	// PHP < 4.1 does not have array_key_exists
	//
	if (phpversion() < '4.1') {
	  eval('function array_key_exists($key, $arr)
             {
                if (!is_array($arr))
                   return false;
                foreach (array_keys($arr) as $k)
                   if ($k == $key)
                      return true;
                return false;
             }');
	}
    }
    //
    // Connect
    //
    function Connect($server="", $port="")
    {
        if ($this->_connected) {
            $this->ErrorState = "HY000";
            $this->ErrorText = "Already connected";
            return FALSE;
        }
        
        if ((strlen($server) == 0) && strlen($this->Server) == 0) {
            $this->ErrorState = "HY000";
            $this->ErrorText = "Server not defined";
            return FALSE;
        }
        if (strlen($server) == 0) $server = $this->Server;
        if (strlen($port) == 0) $port = $this->Port;
        
        $this->_socket = fsockopen($server,
                                   $port,
                                   &$errno,
                                   &$errstr,
                                   30);
        if (!$this->_socket) {
            $this->ErrorState = "HY000";
            $this->ErrorText = "Error in fsockopen() ";
            if ($errno != 0)
                $this->ErrorText .= "before connect - socket create?";
            else
                $this->ErrorText .= "- $errstr";
            return FALSE;
        }
        $this->_connected = TRUE;
        return TRUE;
    }
    //
    // Disconnect
    //
    function Disconnect()
    {
        if (!$this->_connected) {
            $this->ErrorState = "HY000";
            $this->ErrorText = "Not connected";
            return FALSE;
        }
        $this->_connected = FALSE;
        if (!fclose($this->_socket)) {
            $this->ErrorState = "HY000";
            $this->ErrorText = "Error closing socket";
            return FALSE;
        }
        return TRUE;
    }
    //
    // Execute
    //
    function Execute($connectstring = "", $sql = "") 
    {
        //
        // Checks
        //
        if (!$this->_connected) {
            $this->ErrorState = "HY000";
            $this->ErrorText = "Not connected";
            return FALSE;
        }
        if ((strlen($connectstring) == 0) &&
            (strlen($this->ConnectionString) == 0)) {
            $this->ErrorState = "HY000";
            $this->ErrorText = "Missing connection string";
            return FALSE;
        }
        if (strlen($connectstring) == 0)
            $connectstring = $this->ConnectionString;
        
        if ((strlen($sql) == 0) &&
            (!is_array($this->SQL) && (strlen($this->SQL) == 0))) {
            $this->ErrorState = "HY000";
            $this->ErrorText = "Missing SQL";
            return FALSE;
        }
        $this->NumRows = 0;
        $this->NumColumns = 0;
        $this->_current_row = array();
        $this->Rows = array();
        
        if (strlen($sql)) 
            $statements[0] = $sql;
        else {
            if (is_array($this->SQL))
                $statements = $this->SQL;
            else
                $statements[0] = $this->SQL;
        }
        
        $send =
            "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\r\n" .
            "<Request>\r\n" .
            "<ConnectionString>$connectstring</ConnectionString>\r\n";
        if (count($statements) > 1) $send .= "<Transaction>\r\n";
        for ($stmt = 0; $stmt < count($statements); $stmt++) {
            if (!preg_match("/<!\[CDATA\[/", $statements[$stmt]))
                $msql = htmlspecialchars($statements[$stmt], ENT_NOQUOTES);
            else
                $msql = $statements[$stmt];
            $send .= "<Sql>$msql</Sql>\r\n";
        }
        if (count($statements) > 1) $send .= "</Transaction>\r\n";

        $send .= "<Format>\r\n" .
            "<Long/>\r\n";
        if ($this->TruncateTrailingBlanks)
            $send .= "<TruncateTrailingBlanks/>\r\n";

        $send .= "</Format>\r\n";
        
        $send .="\r\n</Request>\r\n";
        $this->Request = $send;
        
        fputs($this->_socket, $send);

        while(!feof($this->_socket)) 
        {
            $out = fgets($this->_socket, 1024);
            $this->XML .= $out;
            if (preg_match("/<\/Result>/", $out)) break;
        }
        $this->_executed = TRUE;
        $this->_pos = 0;
        
        if ($this->Parse) {
            //parse the XML
            $this->_parser = xml_parser_create();
            xml_set_object($this->_parser,&$this);
            
            xml_parser_set_option($this->_parser, XML_OPTION_CASE_FOLDING, 0);

            xml_set_element_handler($this->_parser,
                                    "startElement", "endElement");
            xml_set_character_data_handler($this->_parser, "characterData");

            if (!xml_parse($this->_parser, $this->XML)) {
                $this->ErrorState = "HY000";
                $this->ErrorText =
                    sprintf("XML error: %s at line %d",
                            xml_error_string(xml_get_error_code($this->_parser)),
                            xml_get_current_line_number($this->_parser));
            }       
            xml_parser_free($this->_parser);
        }
        
        return TRUE;
    }
    //
    // First
    //
    function First () 
    {
        if (!$this->_connected) {
            $this->ErrorState = "HY000";
            $this->ErrorText = "Not connected";
            return FALSE;
        }
        if (!$this->_executed) {
            $this->ErrorState = "HY000";
            $this->ErrorText = "No SQL executed yet";
            return FALSE;
        }
        if (!$this->Parse) {
            $this->ErrorState = "HY000";
            $this->ErrorText = "First not available unless XML Parsed.";
            return FALSE;
        }
        $this->_pos = 0;
        return $this->Rows[0];
    }
    
    //
    // Next
    //
    function Next ()
    {
        if (!$this->_connected) {
            $this->ErrorState = "HY000";
            $this->ErrorText = "Not connected";
            return FALSE;
        }
        if (!$this->_executed) {
            $this->ErrorState = "HY000";
            $this->ErrorText = "No SQL executed yet";
            return FALSE;
        }
        if (!$this->Parse) {
            $this->ErrorState = "HY000";
            $this->ErrorText = "Second not available unless XML Parsed.";
            return FALSE;
        }
        $this->_pos++;
        if ($this->_pos >= $this->NumRows) return FALSE;
        return $this->Rows[$this->_pos];
    }
    
    //
    // Last
    //
    function Last ()
    {
        if (!$this->_connected) {
            $this->ErrorState = "HY000";
            $this->ErrorText = "Not connected";
            return FALSE;
        }
        if (!$this->_executed) {
            $this->ErrorState = "HY000";
            $this->ErrorText = "No SQL executed yet";
            return FALSE;
        }
        if (!$this->Parse) {
            $this->ErrorState = "HY000";
            $this->ErrorText = "Last not available unless XML Parsed.";
            return FALSE;
        }
        $this->_pos = $this->NumRows;
        return $this->Rows[$this->NumRows - 1];
        
    }
    
    //
    // handler for the start of elements
    //
    function startElement($parser, $name, $attribs)
    {
        $this->_current_element = $name;
        
        // Save the overall State.
        if ($name == "Result") {
            $this->ErrorState = $attribs['State'];
        }
        else if (($name == "Row") || ($name == "Diagnostic")) {
            $this->Rows[$this->NumRows] = array();
        }
        else if ($name == "Column") {
            if ($this->NumRows == 0) $this->NumColumns++;
            $this->_current_column = $attribs['Name'];
        }
        else if (($name == "State") ||
                 ($name == "Native") ||
                 ($name == "Text")) {
            if ($this->NumRows == 0) $this->NumColumns++;
            $this->_current_column = $name;
        }
    }

    //
    // handler for the end of elements
    //
    function endElement($parser, $name)
    {
        
        if (($name == "Row") || ($name == "Diagnostic")) $this->NumRows++;
        
        $this->_current_column = "";
    }

    //
    // handler for character data
    //
    function characterData($parser, $data)
    {
        if ($this->_current_element == "Row") return;
        if ($this->_current_element == "Result") return;
        if ($this->_current_element == "Diagnostic") return;
        if (strlen($this->_current_column) == 0) return;
        
        if (array_key_exists($this->_current_column,
                             $this->Rows[$this->NumRows]))
            $this->Rows[$this->NumRows][$this->_current_column] .= $data;
        else
            $this->Rows[$this->NumRows][$this->_current_column] = $data;
        
    }

}

?>
